class ApiError(BaseException):
    pass


class CurvedHands(BaseException):
    pass


class InternalError(BaseException):
    pass


class AccessDenied(BaseException):
    pass


class CriticalError(BaseException):
    pass
